@@ -13,6 +13,7 @@ import traceback
 from functools import partial
 from io import FileIO, TextIOWrapper
 from logging import StreamHandler
+from typing import Optional
 
 import zmq
 from IPython.core.application import (  # type:ignore[attr-defined]
@@ -131,7 +132,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
     poller = Any()  # don't restrict this even though current pollers are all Threads
     heartbeat = Instance(Heartbeat, allow_none=True)
 
-    context = Any()
+    context: Optional[zmq.Context] = Any()  # type:ignore[assignment]
     shell_socket = Any()
     control_socket = Any()
     debugpy_socket = Any()
@@ -403,7 +404,8 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
             if socket and not socket.closed:
                 socket.close()
         self.log.debug("Terminating zmq context")
-        self.context.term()
+        if self.context:
+            self.context.term()
         self.log.debug("Terminated zmq context")
 
     def log_connection_info(self):
@@ -445,11 +447,11 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
     def init_blackhole(self):
         """redirects stdout/stderr to devnull if necessary"""
         if self.no_stdout or self.no_stderr:
-            blackhole = open(os.devnull, "w")
+            blackhole = open(os.devnull, "w")  # noqa
             if self.no_stdout:
-                sys.stdout = sys.__stdout__ = blackhole
+                sys.stdout = sys.__stdout__ = blackhole  # type:ignore
             if self.no_stderr:
-                sys.stderr = sys.__stderr__ = blackhole
+                sys.stderr = sys.__stderr__ = blackhole  # type:ignore
 
     def init_io(self):
         """Redirect input streams and set a display hook."""
@@ -469,7 +471,6 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
                 sys.stderr.flush()
             sys.stderr = outstream_factory(self.session, self.iopub_thread, "stderr", echo=e_stderr)
             if hasattr(sys.stderr, "_original_stdstream_copy"):
-
                 for handler in self.log.handlers:
                     if isinstance(handler, StreamHandler) and (handler.stream.buffer.fileno() == 2):
                         self.log.debug("Seeing logger to stderr, rerouting to raw filedescriptor.")
